#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2014 uralbash <root@uralbash.ru>
#
# Distributed under terms of the MIT license.

"""
Views for pages
"""
from pyramid.httpexceptions import HTTPNotFound
from pyramid.renderers import render_to_response
from pyramid.response import Response


class PageView(object):

    def __init__(self, context, request):
        self.request = request
        self.context = context
        self.page = context.node

        if not self.page.visible:
            raise HTTPNotFound

    def page_with_redirect(self):

        if all([hasattr(self.page, attr)
                for attr in ('redirect', 'redirect_url', 'redirect_page')]):
            # Prohibit redirect both url and page
            if self.page.redirect_url and self.page.redirect_page:
                raise HTTPNotFound

            # check redirect type
            if not self.page.redirect_type and self.page.redirect_url:
                redirect_type = '302'
            elif not self.page.redirect_type and self.page.redirect_page:
                redirect_type = '200'
            else:
                redirect_type = str(self.page.redirect_type)

            # Prohibit redirect itself
            if self.page.redirect == self.page and redirect_type != '200':
                raise HTTPNotFound

            # Redirect to Page
            if self.page.redirect_page:
                if not self.page.redirect.visible:
                    raise HTTPNotFound
                if redirect_type == '200':
                    self.page = self.page.redirect
                    return render_to_response(
                        getattr(self.page,
                                'pyramid_pages_template',
                                self.context.template),
                        {'page': self.page},
                        request=self.request
                    )
                else:
                    prefix = self.context.get_prefix(self.request,
                                                     node=self.page.redirect)
                    redirect_resource_url = self.request.resource_url(
                        self.context.__class__(self.page.redirect, prefix))
                    return Response(status_code=int(redirect_type),
                                    location=redirect_resource_url)
            # Redirect to URL
            if self.page.redirect_url:
                if redirect_type == '200':
                    raise HTTPNotFound
                return Response(status_code=int(redirect_type),
                                location=self.page.redirect_url)
        return {'page': self.page}



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2014 uralbash <root@uralbash.ru>
#
# Distributed under terms of the MIT license.

"""
Routes for sacrud_pages
"""
import re

from pyramid.httpexceptions import HTTPNotFound
from sqlalchemy import or_

from .views import PageView
from .security import PREFIX_PAGE, HOME_PAGE

CONFIG_MODELS = 'pyramid_pages.models'
CONFIG_DBSESSION = 'pyramid_pages.dbsession'


class PageResource(object):

    template = 'pyramid_pages/index.jinja2'
    view = PageView
    attr = 'page_with_redirect'

    def __init__(self, node, prefix=None):
        self.node = node
        self.prefix = prefix

    @property
    def __name__(self):
        if self.node and not self.node.slug == '/':
            return self.node.slug
        elif self.node and self.node.slug == '/':
            return ''
        elif self.prefix:
            return self.prefix
        return None

    @property
    def __parent__(self):
        if hasattr(self.node, 'parent'):
            return self.__class__(self.node.parent, self.prefix)
        elif self.node:
            return self.__class__(None, self.prefix)
        return None

    def __getitem__(self, name):
        children = {str(child.slug or ''): self.__class__(child, self.prefix)
                    for child in self.node.children}
        return children[name]

    def __resource_url__(self, request, info):
        separator = '/' if self.prefix else ''
        # XXX: I feel a dissonance here
        info['virtual_path'] = re.sub('/+', '/', info['virtual_path'])
        url = info['app_url'] + separator + info['virtual_path']
        return url

    def __repr__(self):
        return "<{}>".format(self.node.name.encode('utf-8'))

    def get_prefix(self, request, node=None):
        if not node:
            node = self.node
        node = node.__class__
        settings = request.registry.settings
        models = settings[CONFIG_MODELS]
        reversed_models = dict(zip(models.values(), models.keys()))
        prefix = reversed_models.get(node, None)
        if prefix:
            return prefix

        # search prefix for resource
        for model in models.values():
            if not hasattr(model, '__table__') and hasattr(model, 'model')\
                    and model.model == node:
                return reversed_models.get(model, None)
        return None


def page_factory(request):
    settings = request.registry.settings
    models = settings[CONFIG_MODELS]
    prefix = request.matchdict['prefix']
    dbsession = settings[CONFIG_DBSESSION]

    if prefix not in models:
        # prepend {prefix} to *traverse
        request.matchdict['traverse'] =\
            tuple([prefix] + list(request.matchdict['traverse']))
        prefix = None
        table = models['']
    else:
        table = models[prefix]

    if not hasattr(table, '__table__') and hasattr(table, 'model'):
        resource = table
        table = table.model
    else:
        resource = PageResource

    nodes = dbsession.query(table)
    if hasattr(table, 'parent_id'):
        nodes = nodes.filter(or_(table.parent_id.is_(''),
                                 table.parent_id.is_(None),
                                 table.parent.has(table.slug == '/')))
    return {node.slug: resource(node, prefix)
            for node in nodes if node.slug}


def home_page_factory(request):
    settings = request.registry.settings
    models = settings[CONFIG_MODELS]
    table = models[''] or models['/']
    dbsession = settings[CONFIG_DBSESSION]
    node = dbsession.query(table).filter(table.slug.is_('/')).first()
    if not node:
        raise HTTPNotFound
    return PageResource(node)


def register(*args):
    config = args[0]
    settings = config.get_settings()
    models = settings[CONFIG_MODELS]
    for resource in models.values():
        if hasattr(resource, '__table__')\
                and not hasattr(resource, 'model'):
            continue
        resource.model.pyramid_pages_template = resource.template
        config.add_view(resource.view,
                        attr=resource.attr,
                        route_name=PREFIX_PAGE,
                        renderer=resource.template,
                        context=resource,
                        permission=PREFIX_PAGE)


def includeme(config):
    # Home page factory
    config.add_route(HOME_PAGE, '/', factory=home_page_factory)
    config.add_view(PageResource.view,
                    attr=PageResource.attr,
                    route_name=HOME_PAGE,
                    renderer=PageResource.template,
                    context=PageResource,
                    permission=HOME_PAGE)

    # Default page factory
    config.add_route(PREFIX_PAGE, '/{prefix}*traverse', factory=page_factory)
    config.add_view(PageResource.view,
                    attr=PageResource.attr,
                    route_name=PREFIX_PAGE,
                    renderer=PageResource.template,
                    context=PageResource,
                    permission=PREFIX_PAGE)

    import pkg_resources
    pyramid_version = pkg_resources.get_distribution("pyramid").parsed_version
    if pyramid_version >= pkg_resources.SetuptoolsVersion('1.6a1'):
        # Allow you to change settings after including the addon
        config.action('pyramid_pages_routes', register, args=(config, ))
    else:
        config.include(register)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2014 uralbash <root@uralbash.ru>
#
# Distributed under terms of the MIT license.

"""
Models for page.
"""
from sqlalchemy import (Boolean, Column, ForeignKey, Integer, String,
                        UnicodeText)
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy.orm import foreign, relationship
from sqlalchemy.orm.session import Session

from sacrud.common import TableProperty, ClassProperty
from sacrud.exttype import ChoiceType, SlugType
from sqlalchemy_mptt import BaseNestedSets

from .common import Menu

REDIRECT_CHOICES = (
    ('200', 'OK (200)'),
    ('301', 'Moved Permanently (301)'),
    ('302', 'Moved Temporarily (302)'),
)


class PageMixin(object):

    name = Column(String, nullable=False)
    visible = Column(Boolean)
    in_menu = Column(Boolean)
    slug = Column(SlugType('name', False), unique=True, nullable=False)
    description = Column(UnicodeText)

    def __repr__(self):
        return self.name or ''

    def __json__(self, request):
        return self.id


class MpttPageMixin(BaseNestedSets, PageMixin):

    menu_template = 'pyramid_pages/menu/mptt.jinja2'

    def get_menu(self, **kwargs):
        table = self.__class__
        session = Session.object_session(self)
        return Menu(session, table).mptt(**kwargs)


class FlatPageMixin(PageMixin):

    menu_template = 'pyramid_pages/menu/flat.jinja2'

    def get_menu(self):
        table = self.__class__
        session = Session.object_session(self)
        return Menu(session, table).flat()


class RecursionPageMixin(PageMixin):
    """ model with single parent_id field """
    pass


class SeoMixin(object):

    seo_title = Column(String)
    seo_keywords = Column(String)
    seo_description = Column(String)
    seo_metatags = Column(UnicodeText)


class RedirectMixin(object):

    redirect_url = Column(String)
    redirect_type = Column(ChoiceType(choices=REDIRECT_CHOICES))

    @declared_attr
    def redirect_page(cls):
        return Column(
            Integer,
            ForeignKey(
                '{}.{}'.format(cls.__tablename__, cls.get_pk_name()),
                ondelete='CASCADE')
        )

    @declared_attr
    def redirect(cls):
        pk = getattr(cls, cls.get_pk_name())
        return relationship(
            cls, foreign_keys=[cls.redirect_page],
            remote_side='{}.{}'.format(cls.__name__, cls.get_pk_name()),
            primaryjoin=lambda: foreign(cls.redirect_page) == pk,
        )


class BaseSacrudMpttPage(SeoMixin, RedirectMixin, MpttPageMixin):

    # pyramid_sacrud options.
    verbose_name = 'MPTT pages'

    @TableProperty
    def sacrud_css_class(cls):
        col = cls.columns
        return {
            'tinymce': [col.description],
            'description': [col.description],
            'name': [col.name]
        }

    @ClassProperty
    def sacrud_detail_col(cls):
        col = cls.__table__.columns
        return [
            ('', [col.name, col.slug, col.description, col.visible,
                  col.in_menu, cls.parent]),
            ('Redirection', [col.redirect_url, cls.redirect,
                             col.redirect_type]),
            ('SEO', [col.seo_title, col.seo_keywords, col.seo_description,
                     col.seo_metatags])
        ]



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2014 uralbash <root@uralbash.ru>
#
# Distributed under terms of the MIT license.


def includeme(config):
    config.include('.assets')
    config.include('.routes')



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2014 uralbash <root@uralbash.ru>
#
# Distributed under terms of the MIT license.
"""
Menu generator
"""
from collections import OrderedDict


class PageMenu(object):

    def __init__(self, items, template):
        self.items = items
        self.template = template

    def __iter__(self):
        for item in self.items:
            yield item


def sort_by_left(tree):
    """ Sort dict by left and tree_id field
    """
    return OrderedDict(sorted(tree.items(),
                              key=lambda x: (x[0].left, x[0].tree_id)))


def recursive_node_to_dict(node, menu, json=None, json_fields=None):
    result = {}
    children = {}
    if node in menu:
        result = {node: {}}
    for item in node.children:
        children.update(recursive_node_to_dict(item, menu))
    if node in menu:
        result[node] = sort_by_left(children)
    return result


class Menu(object):

    def __init__(self, session, model):
        self.model = model
        self.template = self.model.menu_template
        self.items = session.query(model).filter_by(visible=True)\
            .filter_by(in_menu=True).filter(model.slug != '/')

    def flat(self):
        return PageMenu(self.items, self.template)

    def mptt(self, from_lvl=1, **kwargs):
        menu = self.items.filter(self.model.level >= from_lvl)
        if 'to_lvl' in kwargs:
            to_lvl = int(kwargs['to_lvl'])
            if to_lvl > 0:
                menu = menu.filter(self.model.level <= to_lvl)
            elif to_lvl < 0:
                from sqlalchemy import func
                menu = menu.filter(
                    self.model.level <=
                    func.max(self.model.level + to_lvl).select()
                )
        if 'trees' in kwargs:
            menu = menu.filter(self.model.tree_id.in_(kwargs['trees']))
        if not menu:
            return {}

        tree = {}
        min_lvl = min(menu, key=lambda item: item.level).level
        top_nodes = filter(lambda item: item.level == min_lvl, menu)
        for item in top_nodes:
            tree.update(recursive_node_to_dict(item, menu))

        return PageMenu(sort_by_left(tree).items(), self.template)



import os

from setuptools import find_packages, setup

this = os.path.dirname(os.path.realpath(__file__))


def read(name):
    with open(os.path.join(this, name)) as f:
        return f.read()

setup(
    name='pyramid_pages',
    version='0.0.2',
    url='http://github.com/ITCase/pyramid_pages/',
    author='Svintsov Dmitry',
    author_email='sacrud@uralbash.ru',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    test_suite="nose.collector",
    license="MIT",
    description='Tree pages for pyramid',
    long_description=read('README.rst'),
    install_requires=read('requirements.txt'),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Framework :: Pyramid ",
        "Topic :: Internet",
        "Topic :: Database",
    ],
)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2014 uralbash <root@uralbash.ru>
#
# Distributed under terms of the MIT license.

"""
Assets
"""


def includeme(config):
    config.include('pyramid_jinja2')
    config.add_jinja2_extension('jinja2.ext.with_')
    config.add_jinja2_search_path("pyramid_pages:templates")
    config.add_static_view('pyramid_pages_static', 'pyramid_pages:static')



from pyramid.httpexceptions import HTTPNotFound

from pyramid_pages.routes import PageResource
from pyramid_pages.views import PageView

from . import NewsPage, UnitTestBase, WebPage, settings
from pyramid_pages_example import Gallery


class TestResource(UnitTestBase):
    pass


class TestCommon(UnitTestBase):
    pass


class TestPageView(UnitTestBase):

    """ VISIBLE
    """

    def test_visible_node(self):
        """ Node is visible.
        """

        def do_it(model):
            node = model(visible=True)
            context = PageResource(node)
            view = PageView(context, self.request).page_with_redirect()
            self.assertEqual(view['page'], node)

        do_it(NewsPage)
        do_it(WebPage)

    def test_not_visible_node(self):
        """ Node is not visible.
        """

        def do_it(model):
            # visible is None
            node = model()
            context = PageResource(node)
            self.assertRaises(HTTPNotFound, PageView, context, self.request)

            # visible is False
            node = model(visible=False)
            context = PageResource(node)
            self.assertRaises(HTTPNotFound, PageView, context, self.request)

        do_it(NewsPage)
        do_it(WebPage)

    """ REDIRECT

    redirect_url: URI string e.g.: http://example.com/
    redirect_page: node of tree
    redirect_type: [200, 301, 302]
    """
    def test_redirect_200(self):
        """ Redirect 200 w/o page.

        redirect_type = 200
        redirect_url  = None
        redirect_page = None
        """
        node = WebPage(visible=True, redirect_type=200)
        context = PageResource(node)
        view = PageView(context, self.request).page_with_redirect()
        self.assertEqual(view['page'], node)

    """ 200 OK
        redirect to page
    """
    def test_redirect_200_to_self(self):
        """ Redirect 200 to itself.

        redirect_type = 200
        redirect_url  = None
        redirect_page = self
        """
        node = WebPage(
            id=1, visible=True,
            redirect_type=200, redirect_page=1
        )
        node.redirect = node
        context = PageResource(node)
        context.template = 'json'
        response = PageView(context, self.request).page_with_redirect()
        self.assertEqual(response.json['page'], 1)

    def test_redirect_page_with_out_type(self):
        """ Redirect to page w/o redirect type.

        redirect_type = None
        redirect_url  = None
        redirect_page = 2
        """
        node2 = WebPage(id=2)
        node = WebPage(
            id=1, visible=True,
            redirect_page=2, redirect=node2
        )
        context = PageResource(node)
        view = PageView(context, self.request)
        self.assertRaises(HTTPNotFound, view.page_with_redirect)

    def test_redirect_200_to_not_visible_page(self):
        """ Redirect 200 to not visible page.

        redirect_type = 200
        redirect_url  = None
        redirect_page = 2
        """
        node2 = WebPage(id=2)
        node = WebPage(
            id=1, visible=True,
            redirect_type=200, redirect_page=2, redirect=node2
        )
        context = PageResource(node)
        view = PageView(context, self.request)
        self.assertRaises(HTTPNotFound, view.page_with_redirect)

    def test_redirect_200_to_visible_page(self):
        """ Redirect 200 to visible page.

        redirect_type = 200
        redirect_url  = None
        redirect_page = 2
        """
        node2 = WebPage(id=2, visible=True)
        node = WebPage(
            id=1, visible=True,
            redirect_type=200, redirect_page=2, redirect=node2
        )
        context = PageResource(node)
        context.template = 'json'
        response = PageView(context, self.request).page_with_redirect()
        self.assertEqual(response.json['page'], 2)

    """ 300
        redirect to page
    """
    def test_redirect_300_to_self(self):
        """ Redirect 300 to itself.

        redirect_type = 300
        redirect_url  = None
        redirect_page = self
        """
        def do_it(redirect_code):
            node = WebPage(
                id=1, visible=True,
                redirect_type=redirect_code, redirect_page=1
            )
            node.redirect = node
            context = PageResource(node)
            view = PageView(context, self.request)
            self.assertRaises(HTTPNotFound, view.page_with_redirect)

        do_it(301)
        do_it(302)

    def test_redirect_300_to_not_visible_page(self):
        """ Redirect 300 to not visible page.

        redirect_type = 300
        redirect_url  = None
        redirect_page = 2
        """
        def do_it(redirect_code):
            node2 = WebPage(id=2)
            node = WebPage(
                id=1, visible=True,
                redirect_type=redirect_code, redirect_page=2, redirect=node2
            )
            context = PageResource(node)
            view = PageView(context, self.request)
            self.assertRaises(HTTPNotFound, view.page_with_redirect)

        do_it(301)
        do_it(302)

    def test_redirect_300_to_visible_page(self):
        """ Redirect 300 to visible page.

        redirect_type = 300
        redirect_url  = None
        redirect_page = 2
        """
        self.request.registry.settings = settings

        def do_it(redirect_code):
            self.drop_db()
            self.create_db()

            node2 = WebPage(id=2, visible=True, name='node2', slug='node2')
            node1 = WebPage(
                id=1, visible=True, name='node', slug='node',
                redirect_type=redirect_code, redirect=node2
            )
            node3 = WebPage(id=3, visible=True, name='node3', slug='node3',
                            parent_id=2)
            node4 = WebPage(
                id=4, visible=True, name='node4', slug='node4',
                redirect_type=redirect_code, redirect=node3
            )
            node5_inheritance = Gallery(
                id=5, visible=True, name='node5', slug='node5')
            node6 = WebPage(
                id=6, visible=True, name='node6', slug='node6',
                redirect_type=redirect_code, redirect=node5_inheritance
            )

            self.dbsession.add(node1)
            self.dbsession.add(node2)
            self.dbsession.flush()
            self.dbsession.add(node3)
            self.dbsession.add(node4)
            self.dbsession.add(node5_inheritance)
            self.dbsession.add(node6)
            self.dbsession.commit()

            # 301
            context = PageResource(node1)
            view = PageView(context, self.request).page_with_redirect()
            self.assertEqual(view.status_code, redirect_code)
            self.assertEqual(view.location, 'http://example.com/pages/node2/')

            context = PageResource(node4)
            view = PageView(context, self.request).page_with_redirect()
            self.assertEqual(view.status_code, redirect_code)
            self.assertEqual(view.location,
                             'http://example.com/pages/node2/node3/')

            context = PageResource(node6)
            view = PageView(context, self.request).page_with_redirect()
            self.assertEqual(view.status_code, redirect_code)
            self.assertEqual(view.location,
                             'http://example.com/gallery/node5/')
            self.dbsession.close()

        do_it(301)
        do_it(302)

    """ 200 OK
        redirect to URL
    """
    def test_redirect_200_to_url(self):
        """ Redirect 200 to external URL.

        redirect_type = 200
        redirect_url  = http://example.org
        redirect_page = None
        """
        URL = 'http://example.org'
        node = WebPage(
            visible=True, redirect_type=200, redirect_url=URL
        )
        context = PageResource(node)
        view = PageView(context, self.request)
        self.assertRaises(HTTPNotFound, view.page_with_redirect)

    def test_redirect_url_with_out_type(self):
        """ Redirect to external URL w/o type.

        redirect_type = None
        redirect_url  = http://example.org
        redirect_page = None
        """
        URL = 'http://example.org'
        node = WebPage(
            visible=True, redirect_url=URL,
        )

        context = PageResource(node)
        view = PageView(context, self.request).page_with_redirect()
        self.assertEqual(view.status_code, 302)
        self.assertEqual(view.location, 'http://example.org')

    """ 300
        redirect to URL
    """
    def test_redirect_300_to_url(self):
        """ Redirect 300 to external URL.

        redirect_type = 300
        redirect_url  = http://example.org
        redirect_page = None
        """
        def do_it(redirect_code):
            URL = 'http://example.org'
            node = WebPage(
                visible=True, redirect_type=301, redirect_url=URL
            )

            # 301
            context = PageResource(node)
            view = PageView(context, self.request).page_with_redirect()
            self.assertEqual(view.status_code, 301)
            self.assertEqual(view.location, 'http://example.org')

        do_it(301)
        do_it(302)

    """ Bad case.
    """
    def test_redirect_simultaneously_url_and_page(self):
        """ Redirect simultaneously to URL and Page.

        redirect_type = None
        redirect_url  = http://example.org
        redirect_page = 2
        """
        URL = 'http://example.org'
        node2 = WebPage(id=2, visible=True)
        node = WebPage(
            visible=True, redirect_url=URL,
            redirect_page=2, redirect=node2
        )

        context = PageResource(node)
        view = PageView(context, self.request)
        self.assertRaises(HTTPNotFound, view.page_with_redirect)



from . import IntegrationTestBase


class BasePageView(object):

    def test_get_root_page(self):
        self.app.get('/', status=200)

    def test_get_page_from_tree(self):
        self.app.get('/pages/about-company/', status=200)
        self.app.get('/about-company/', status=200)

    def test_redirect_301_page(self):
        self.app.get('/pages/about-company/redirect-301/',
                     status=301)

    def test_redirect_302_page(self):
        self.app.get('/pages/about-company/redirect-301/kompania-itcase/',
                     status=302)

    def test_bad_page_source(self):
        self.app.get('/I-would-like-to-be-a-page-but-Im-just-a-404-error',
                     status=404)


class TestPageView(IntegrationTestBase, BasePageView):
    pass



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2015 uralbash <root@uralbash.ru>
#
# Distributed under terms of the MIT license.

"""
Base classes for tests
http://www.sontek.net/blog/2011/12/01/writing_tests_for_pyramid_and_sqlalchemy.html
"""
import unittest

from pyramid import testing
from sqlalchemy import engine_from_config
from sqlalchemy.orm import sessionmaker
from webtest import TestApp
from sqlalchemy_mptt import mptt_sessionmaker

import imp
imp.load_source('pyramid_pages_example', 'example/pyramid_pages_example.py')

from pyramid_pages_example import NewsPage, WebPage, Base, main, models  # noqa

settings = {
    'sqlalchemy.url': 'sqlite:///test.sqlite',
    'pyramid_pages.models': models
}


class BaseTestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.engine = engine_from_config(settings, prefix='sqlalchemy.')
        cls.DBSession = mptt_sessionmaker(sessionmaker())

    def setUp(self):
        # bind an individual Session to the connection
        self.dbsession = self.DBSession(bind=self.engine)
        self.create_db()

    def tearDown(self):
        # rollback - everything that happened with the
        # Session above (including calls to commit())
        # is rolled back.
        testing.tearDown()
        self.drop_db()
        self.dbsession.close()

    def drop_db(self):
        Base.metadata.drop_all(bind=self.engine)
        self.dbsession.commit()

    def create_db(self):
        Base.metadata.create_all(bind=self.engine)
        self.dbsession.commit()


class UnitTestBase(BaseTestCase):

    def setUp(self):
        self.request = testing.DummyRequest()
        self.config = testing.setUp(request=self.request)
        super(UnitTestBase, self).setUp()


class IntegrationTestBase(BaseTestCase):

    def setUp(self):
        self.app = TestApp(main({}, **settings))
        self.config = testing.setUp()
        super(IntegrationTestBase, self).setUp()
